#include "kcolor.h"

#include <QtGui>

// #define TEST_BLEND

class Widget : public QWidget
{
    Q_OBJECT
public:
    Widget(QWidget *parent=0);

    QSize sizeHint() const;
protected:
    void paintEvent(QPaintEvent *e);

    void drawQColorBand(QPainter *p, const QColor &clr);
    void drawKColorBand(QPainter *p, const QColor &clr);
private:
    QColor m_color1;
    QColor m_color2;
    QColor m_color3;
    QColor m_color4;
};

static const int bandWidth = 30;

Widget::Widget(QWidget *parent )
    : QWidget(parent)
{
    m_color1 = QColor::fromRgb(128,64,64);
    m_color2 = QColor::fromRgb(64,128,64);
    m_color3 = QColor::fromRgb(64,64,128);
    m_color4 = QColor::fromRgb(128,128,128);
}


void Widget::paintEvent(QPaintEvent *e)
{
    QPainter p(this);
    p.setClipRect(e->rect());

    p.setWindow(0, 0, 320, 200);

    p.save();
    drawQColorBand(&p, m_color1);
    p.translate(35, 0);
    drawKColorBand(&p, m_color1);
    p.restore();

    p.translate(80, 0);

    p.save();
    drawQColorBand(&p, m_color2);
    p.translate(35, 0);
    drawKColorBand(&p, m_color2);
    p.restore();

    p.translate(80, 0);

    p.save();
    drawQColorBand(&p, m_color3);
    p.translate(35, 0);
    drawKColorBand(&p, m_color3);
    p.restore();

    p.translate(80, 0);

    p.save();
    drawQColorBand(&p, m_color4);
    p.translate(35, 0);
    drawKColorBand(&p, m_color4);
    p.restore();
}

void Widget::drawQColorBand(QPainter *p, const QColor &clr)
{
    const int max = 250;
    const int h   = 100;
    const int diff = int(max/qreal(h));

    p->save();
    for (int i = 1; i <= 200; ++i) {
        int val;
        if (i <= 100)
            val = i;
        else
            val = int(((i - 100) * diff) + 100);
        QColor c = clr.lighter(val);
        p->setPen(QPen(c, 1));
        p->drawLine(0, i, bandWidth, i);
        if (i == 100) {
            p->setPen(QPen(QColor(0, 0, 0), 1));
            p->drawLine(bandWidth-5, i, bandWidth+2, i);
        }
    }
    p->restore();
}

QSize Widget::sizeHint() const
{
    return QSize(600, 400);
}

void Widget::drawKColorBand(QPainter *p, const QColor &clr)
{
    KColor kclr(clr);
    p->save();
    for (int i = 1; i <= 200; ++i) {
        KColor kc;
        if (i <= 100) {
            qreal d = qreal(i)/100.;
            kc = kclr.darken(d, KColor::Hsl);
        } else {
            qreal d = 1 - qreal(i-100)/100.;
            kc = kclr.lighten(d, KColor::Hsl);
        }
        QColor c = kc;
        p->setPen(QPen(c, 1));
        p->drawLine(0, i, bandWidth, i);

        if (i == 100){
            p->setPen(QPen(QColor(0, 0, 0), 1));
            p->drawLine(bandWidth-5, i, bandWidth+2, i);
        }
    }
    p->restore();
}

#include "main.moc"


inline QColor blendColor(const QColor &one,
                         const QColor &two,
                         QPainter::CompositionMode comp = QPainter::CompositionMode_SourceOver)
{
    QImage img(1, 1, QImage::Format_ARGB32_Premultiplied);
    img.fill(0);
    QPainter p(&img);
    p.fillRect(0, 0, 1, 1, one);
    p.setCompositionMode(comp);
    p.fillRect(0, 0, 1, 1, two);
    p.end();
    return img.pixel(0, 0);
}

int main(int argc, char **argv)
{
    QApplication app(argc, argv);

#ifdef TEST_BLEND
    QTime time;

    QColor clr1(255, 0, 0, 127);
    QColor clr2(0, 255, 0, 127);
    QColor clr;

    time.restart();
    for (int i = 0; i < 10000; ++i) {
        fprintf(stderr, "xxxxxxxxxxx = %d\n", clr1.rgba());
    }
    int elapsed = time.elapsed();
    qDebug()<<"10000 fprintf's took = "<<elapsed;

    time.restart();
    for (int i = 0; i < 10000; ++i) {
        clr = blendColor(clr1, clr2);
    }
    elapsed = time.elapsed();
    qDebug()<<"10000 blendColor's took = "<<elapsed;

    time.restart();
    for (int i = 0; i < 10000; ++i) {
        clr = KColor::blend(clr1, clr2);
    }
    elapsed = time.elapsed();
    qDebug()<<"10000 KColor::blend's took = "<<elapsed;

    return 0;
#else
    Widget w;
    w.show();
    return app.exec();
#endif
}
